@@ -41,9 +41,9 @@ void ReportWarningHelper::Report(const char* aMessageName,
   mLoader->ReportWarningToConsole(mRequest, aMessageName, aParams);
 }
 
-// https://wicg.github.io/import-maps/#parse-a-url-like-import-specifier
-static already_AddRefed<nsIURI> ParseURLLikeImportSpecifier(
-    const nsAString& aSpecifier, nsIURI* aBaseURL) {
+// https://html.spec.whatwg.org/multipage/webappapis.html#resolving-a-url-like-module-specifier
+static ResolveResult ResolveURLLikeModuleSpecifier(const nsAString& aSpecifier,
+                                                   nsIURI* aBaseURL) {
   nsCOMPtr<nsIURI> uri;
   nsresult rv;
 
@@ -56,25 +56,25 @@ static already_AddRefed<nsIURI> ParseURLLikeImportSpecifier(
     rv = NS_NewURI(getter_AddRefs(uri), aSpecifier, nullptr, aBaseURL);
     // Step 1.2. If url is failure, then return null.
     if (NS_FAILED(rv)) {
-      return nullptr;
+      return Err(ResolveError::Failure);
     }
 
     // Step 1.3. Return url.
-    return uri.forget();
+    return WrapNotNull(uri);
   }
 
   // Step 2. Let url be the result of parsing specifier (with no base URL).
   rv = NS_NewURI(getter_AddRefs(uri), aSpecifier);
   // Step 3. If url is failure, then return null.
   if (NS_FAILED(rv)) {
-    return nullptr;
+    return Err(ResolveError::FailureMayBeBare);
   }
 
   // Step 4. Return url.
-  return uri.forget();
+  return WrapNotNull(uri);
 }
 
-// https://wicg.github.io/import-maps/#normalize-a-specifier-key
+// https://html.spec.whatwg.org/multipage/webappapis.html#normalizing-a-specifier-key
 static void NormalizeSpecifierKey(const nsAString& aSpecifierKey,
                                   nsIURI* aBaseURL,
                                   const ReportWarningHelper& aWarning,
@@ -90,12 +90,13 @@ static void NormalizeSpecifierKey(const nsAString& aSpecifierKey,
     return;
   }
 
-  // Step 2. Let url be the result of parsing a URL-like import specifier, given
-  // specifierKey and baseURL.
-  nsCOMPtr<nsIURI> url = ParseURLLikeImportSpecifier(aSpecifierKey, aBaseURL);
+  // Step 2. Let url be the result of resolving a URL-like module specifier,
+  // given specifierKey and baseURL.
+  auto parseResult = ResolveURLLikeModuleSpecifier(aSpecifierKey, aBaseURL);
 
   // Step 3. If url is not null, then return the serialization of url.
-  if (url) {
+  if (parseResult.isOk()) {
+    nsCOMPtr<nsIURI> url = parseResult.unwrap();
     aRetVal = NS_ConvertUTF8toUTF16(url->GetSpecOrDefault());
     return;
   }
@@ -104,11 +105,11 @@ static void NormalizeSpecifierKey(const nsAString& aSpecifierKey,
   aRetVal = aSpecifierKey;
 }
 
-// https://wicg.github.io/import-maps/#sort-and-normalize-a-specifier-map
+// https://html.spec.whatwg.org/multipage/webappapis.html#sorting-and-normalizing-a-module-specifier-map
 static UniquePtr<SpecifierMap> SortAndNormalizeSpecifierMap(
     JSContext* aCx, JS::HandleObject aOriginalMap, nsIURI* aBaseURL,
     const ReportWarningHelper& aWarning) {
-  // Step 1. Let normalized be an empty map.
+  // Step 1. Let normalized be an empty ordered map.
   UniquePtr<SpecifierMap> normalized = MakeUnique<SpecifierMap>();
 
   JS::Rooted<JS::IdVector> specifierKeys(aCx, JS::IdVector(aCx));
@@ -138,8 +139,8 @@ static UniquePtr<SpecifierMap> SortAndNormalizeSpecifierMap(
                    nullptr);
     // Step 2.3. If value is not a string, then:
     if (!idVal.isString()) {
-      // Step 2.3.1. Report a warning to the console that addresses need to
-      // be strings.
+      // Step 2.3.1. The user agent may report a warning to the console
+      // indicating that addresses need to be strings.
       aWarning.Report("ImportMapAddressesNotStrings");
 
       // Step 2.3.2. Set normalized[normalizedSpecifierKey] to null.
@@ -152,14 +153,14 @@ static UniquePtr<SpecifierMap> SortAndNormalizeSpecifierMap(
     nsAutoJSString value;
     NS_ENSURE_TRUE(value.init(aCx, idVal), nullptr);
 
-    // Step 2.4. Let addressURL be the result of parsing a URL-like import
+    // Step 2.4. Let addressURL be the result of resolving a URL-like module
     // specifier given value and baseURL.
-    nsCOMPtr<nsIURI> addressURL = ParseURLLikeImportSpecifier(value, aBaseURL);
+    auto parseResult = ResolveURLLikeModuleSpecifier(value, aBaseURL);
 
     // Step 2.5. If addressURL is null, then:
-    if (!addressURL) {
-      // Step 2.5.1. Report a warning to the console that the address was
-      // invalid.
+    if (parseResult.isErr()) {
+      // Step 2.5.1. The user agent may report a warning to the console
+      // indicating that the address was invalid.
       AutoTArray<nsString, 1> params;
       params.AppendElement(value);
       aWarning.Report("ImportMapInvalidAddress", params);
@@ -171,14 +172,16 @@ static UniquePtr<SpecifierMap> SortAndNormalizeSpecifierMap(
       continue;
     }
 
+    nsCOMPtr<nsIURI> addressURL = parseResult.unwrap();
     nsCString address = addressURL->GetSpecOrDefault();
     // Step 2.6. If specifierKey ends with U+002F (/), and the serialization
     // of addressURL does not end with U+002F (/), then:
     if (StringEndsWith(specifierKey, u"/"_ns) &&
         !StringEndsWith(address, "/"_ns)) {
-      // Step 2.6.1. Report a warning to the console that an invalid address
-      // was given for the specifier key specifierKey; since specifierKey
-      // ended in a slash, the address needs to as well.
+      // Step 2.6.1. The user agent may report a warning to the console
+      // indicating that an invalid address was given for the specifier key
+      // specifierKey; since specifierKey ends with a slash, the address needs
+      // to as well.
       AutoTArray<nsString, 2> params;
       params.AppendElement(specifierKey);
       params.AppendElement(NS_ConvertUTF8toUTF16(address));
@@ -227,7 +230,7 @@ static bool IsMapObject(JSContext* aCx, JS::HandleValue aMapVal, bool* aIsMap) {
   return true;
 }
 
-// https://wicg.github.io/import-maps/#sort-and-normalize-scopes
+// https://html.spec.whatwg.org/multipage/webappapis.html#sorting-and-normalizing-scopes
 static UniquePtr<ScopeMap> SortAndNormalizeScopes(
     JSContext* aCx, JS::HandleObject aOriginalMap, nsIURI* aBaseURL,
     const ReportWarningHelper& aWarning) {
@@ -245,9 +248,9 @@ static UniquePtr<ScopeMap> SortAndNormalizeScopes(
     nsAutoJSString scopePrefix;
     NS_ENSURE_TRUE(scopePrefix.init(aCx, scopeKey), nullptr);
 
-    // Step 2.1. If potentialSpecifierMap is not a map, then throw a TypeError
-    // indicating that the value of the scope with prefix scopePrefix needs to
-    // be a JSON object.
+    // Step 2.1. If potentialSpecifierMap is not an ordered map, then throw a
+    // TypeError indicating that the value of the scope with prefix scopePrefix
+    // needs to be a JSON object.
     JS::RootedValue mapVal(aCx);
     NS_ENSURE_TRUE(JS_GetPropertyById(aCx, aOriginalMap, scopeKey, &mapVal),
                    nullptr);
@@ -263,16 +266,16 @@ static UniquePtr<ScopeMap> SortAndNormalizeScopes(
       return nullptr;
     }
 
-    // Step 2.2. Let scopePrefixURL be the result of parsing scopePrefix with
-    // baseURL as the base URL.
+    // Step 2.2. Let scopePrefixURL be the result of URL parsing scopePrefix
+    // with baseURL.
     nsCOMPtr<nsIURI> scopePrefixURL;
     nsresult rv = NS_NewURI(getter_AddRefs(scopePrefixURL), scopePrefix,
                             nullptr, aBaseURL);
 
     // Step 2.3. If scopePrefixURL is failure, then:
     if (NS_FAILED(rv)) {
-      // Step 2.3.1. Report a warning to the console that the scope prefix URL
-      // was not parseable.
+      // Step 2.3.1. The user agent may report a warning to the console that
+      // the scope prefix URL was not parseable.
       AutoTArray<nsString, 1> params;
       params.AppendElement(scopePrefix);
       aWarning.Report("ImportMapScopePrefixNotParseable", params);
@@ -298,14 +301,15 @@ static UniquePtr<ScopeMap> SortAndNormalizeScopes(
                                  std::move(specifierMap));
   }
 
-  // Step 3. Return the result of sorting normalized, with an entry a being less
-  // than an entry b if b’s key is code unit less than a’s key.
+  // Step 3. Return the result of sorting in descending order normalized, with
+  // an entry a being less than an entry b if a's key is code unit less than b's
+  // key.
   //
   // Impl note: The sorting is done when inserting the entry.
   return normalized;
 }
 
-// https://wicg.github.io/import-maps/#parse-an-import-map-string
+// https://html.spec.whatwg.org/multipage/webappapis.html#parse-an-import-map-string
 // static
 UniquePtr<ImportMap> ImportMap::ParseString(
     JSContext* aCx, SourceText<char16_t>& aInput, nsIURI* aBaseURL,
@@ -314,13 +318,31 @@ UniquePtr<ImportMap> ImportMap::ParseString(
   // input.
   JS::Rooted<JS::Value> parsedVal(aCx);
   if (!JS_ParseJSON(aCx, aInput.get(), aInput.length(), &parsedVal)) {
-    // If JS_ParseJSON fail it will throw SyntaxError.
     NS_WARNING("Parsing Import map string failed");
+
+    // If JS_ParseJSON fails we check if it throws a SyntaxError.
+    // If so we update the error message from JSON parser to make it more clear
+    // that the parsing of import map has failed.
+    MOZ_ASSERT(JS_IsExceptionPending(aCx));
+    JS::Rooted<JS::Value> exn(aCx);
+    if (!JS_GetPendingException(aCx, &exn)) {
+      return nullptr;
+    }
+    MOZ_ASSERT(exn.isObject());
+    JS::Rooted<JSObject*> obj(aCx, &exn.toObject());
+    JSErrorReport* err = JS_ErrorFromException(aCx, obj);
+    if (err->exnType == JSEXN_SYNTAXERR) {
+      JS_ClearPendingException(aCx);
+      JS_ReportErrorNumberASCII(aCx, js::GetErrorMessage, nullptr,
+                                JSMSG_IMPORT_MAPS_PARSE_FAILED,
+                                err->message().c_str());
+    }
+
     return nullptr;
   }
 
-  // Step 2. If parsed is not a map, then throw a TypeError indicating that
-  // the top-level value needs to be a JSON object.
+  // Step 2. If parsed is not an ordered map, then throw a TypeError indicating
+  // that the top-level value needs to be a JSON object.
   bool isMap;
   if (!IsMapObject(aCx, parsedVal, &isMap)) {
     return nullptr;
@@ -337,7 +359,7 @@ UniquePtr<ImportMap> ImportMap::ParseString(
     return nullptr;
   }
 
-  // Step 3. Let sortedAndNormalizedImports be an empty map.
+  // Step 3. Let sortedAndNormalizedImports be an empty ordered map.
   //
   // Impl note: If parsed["imports"] doesn't exist, we will allocate
   // sortedAndNormalizedImports to an empty map in Step 8 below.
@@ -345,8 +367,9 @@ UniquePtr<ImportMap> ImportMap::ParseString(
 
   // Step 4. If parsed["imports"] exists, then:
   if (!importsVal.isUndefined()) {
-    // Step 4.1. If parsed["imports"] is not a map, then throw a TypeError
-    // indicating that the "imports" top-level key needs to be a JSON object.
+    // Step 4.1. If parsed["imports"] is not an ordered map, then throw a
+    // TypeError indicating that the "imports" top-level key needs to be a JSON
+    // object.
     bool isMap;
     if (!IsMapObject(aCx, importsVal, &isMap)) {
       return nullptr;
@@ -358,7 +381,7 @@ UniquePtr<ImportMap> ImportMap::ParseString(
     }
 
     // Step 4.2. Set sortedAndNormalizedImports to the result of sorting and
-    // normalizing a specifier map given parsed["imports"] and baseURL.
+    // normalizing a module specifier map given parsed["imports"] and baseURL.
     JS::RootedObject importsObj(aCx, &importsVal.toObject());
     sortedAndNormalizedImports =
         SortAndNormalizeSpecifierMap(aCx, importsObj, aBaseURL, aWarning);
@@ -372,7 +395,7 @@ UniquePtr<ImportMap> ImportMap::ParseString(
     return nullptr;
   }
 
-  // Step 5. Let sortedAndNormalizedScopes be an empty map.
+  // Step 5. Let sortedAndNormalizedScopes be an empty ordered map.
   //
   // Impl note: If parsed["scopes"] doesn't exist, we will allocate
   // sortedAndNormalizedScopes to an empty map in Step 8 below.
@@ -380,8 +403,9 @@ UniquePtr<ImportMap> ImportMap::ParseString(
 
   // Step 6. If parsed["scopes"] exists, then:
   if (!scopesVal.isUndefined()) {
-    // Step 6.1. If parsed["scopes"] is not a map, then throw a TypeError
-    // indicating that the "scopes" top-level key needs to be a JSON object.
+    // Step 6.1. If parsed["scopes"] is not an ordered map, then throw a
+    // TypeError indicating that the "scopes" top-level key needs to be a JSON
+    // object.
     bool isMap;
     if (!IsMapObject(aCx, scopesVal, &isMap)) {
       return nullptr;
@@ -403,8 +427,8 @@ UniquePtr<ImportMap> ImportMap::ParseString(
   }
 
   // Step 7. If parsed’s keys contains any items besides "imports" or
-  // "scopes", report a warning to the console that an invalid top-level key
-  // was present in the import map.
+  // "scopes", then the user agent should report a warning to the console
+  // indicating that an invalid top-level key was present in the import map.
   JS::Rooted<JS::IdVector> keys(aCx, JS::IdVector(aCx));
   if (!JS_Enumerate(aCx, parsedObj, &keys)) {
     return nullptr;
@@ -432,7 +456,7 @@ UniquePtr<ImportMap> ImportMap::ParseString(
     sortedAndNormalizedScopes = MakeUnique<ScopeMap>();
   }
 
-  // Step 8. Return the import map whose imports are
+  // Step 8. Return an import map whose imports are
   // sortedAndNormalizedImports and whose scopes scopes are
   // sortedAndNormalizedScopes.
   return MakeUnique<ImportMap>(std::move(sortedAndNormalizedImports),
@@ -448,7 +472,7 @@ static bool IsSpecialScheme(nsIURI* aURI) {
          scheme.EqualsLiteral("ws") || scheme.EqualsLiteral("wss");
 }
 
-// https://wicg.github.io/import-maps/#resolve-an-imports-match
+// https://html.spec.whatwg.org/multipage/webappapis.html#resolving-an-imports-match
 static mozilla::Result<nsCOMPtr<nsIURI>, ResolveError> ResolveImportsMatch(
     nsString& aNormalizedSpecifier, nsIURI* aAsURL,
     const SpecifierMap* aSpecifierMap) {
@@ -481,7 +505,7 @@ static mozilla::Result<nsCOMPtr<nsIURI>, ResolveError> ResolveImportsMatch(
 
     // Step 1.2. If all of the following are true:
     // specifierKey ends with U+002F (/),
-    // normalizedSpecifier starts with specifierKey, and
+    // specifierKey is a code unit prefix of normalizedSpecifier, and
     // either asURL is null, or asURL is special
     if (StringEndsWith(specifierKey, u"/"_ns) &&
         StringBeginsWith(aNormalizedSpecifier, specifierKey) &&
@@ -507,12 +531,12 @@ static mozilla::Result<nsCOMPtr<nsIURI>, ResolveError> ResolveImportsMatch(
       nsAutoString afterPrefix(
           Substring(aNormalizedSpecifier, specifierKey.Length()));
 
-      // Step 1.2.4. Assert: resolutionResult, serialized, ends with "/", as
-      // enforced during parsing.
+      // Step 1.2.4. Assert: resolutionResult, serialized, ends with U+002F (/),
+      // as enforced during parsing
       MOZ_ASSERT(StringEndsWith(resolutionResult->GetSpecOrDefault(), "/"_ns));
 
-      // Step 1.2.5. Let url be the result of parsing afterPrefix relative to
-      // the base URL resolutionResult.
+      // Step 1.2.5. Let url be the result of URL parsing afterPrefix with
+      // resolutionResult.
       nsCOMPtr<nsIURI> url;
       nsresult rv = NS_NewURI(getter_AddRefs(url), afterPrefix, nullptr,
                               resolutionResult);
@@ -539,8 +563,8 @@ static mozilla::Result<nsCOMPtr<nsIURI>, ResolveError> ResolveImportsMatch(
       // Step 1.2.7. Assert: url is a URL.
       MOZ_ASSERT(url);
 
-      // Step 1.2.8. If the serialization of url does not start with the
-      // serialization of resolutionResult, then throw a TypeError indicating
+      // Step 1.2.8. If the serialization of resolutionResult is not a code unit
+      // prefix of the serialization of url, then throw a TypeError indicating
       // that resolution of normalizedSpecifier was blocked due to it
       // backtracking above its prefix specifierKey.
       //
@@ -568,7 +592,7 @@ static mozilla::Result<nsCOMPtr<nsIURI>, ResolveError> ResolveImportsMatch(
   return nsCOMPtr<nsIURI>(nullptr);
 }
 
-// https://wicg.github.io/import-maps/#resolve-a-module-specifier
+// https://html.spec.whatwg.org/multipage/webappapis.html#resolve-a-module-specifier
 // static
 ResolveResult ImportMap::ResolveModuleSpecifier(ImportMap* aImportMap,
                                                 ScriptLoaderInterface* aLoader,
@@ -577,36 +601,41 @@ ResolveResult ImportMap::ResolveModuleSpecifier(ImportMap* aImportMap,
   LOG(("ImportMap::ResolveModuleSpecifier specifier: %s",
        NS_ConvertUTF16toUTF8(aSpecifier).get()));
   nsCOMPtr<nsIURI> baseURL;
-  if (aScript) {
+  if (aScript && !aScript->IsEventScript()) {
     baseURL = aScript->BaseURL();
   } else {
     baseURL = aLoader->GetBaseURI();
   }
 
-  // Step 6. Let asURL be the result of parsing a URL-like import specifier
+  // Step 7. Let asURL be the result of resolving a URL-like module specifier
   // given specifier and baseURL.
   //
-  // Impl note: Step 5 is done below if aImportMap exists.
-  nsCOMPtr<nsIURI> asURL = ParseURLLikeImportSpecifier(aSpecifier, baseURL);
+  // Impl note: Step 6 is done below if aImportMap exists.
+  auto parseResult = ResolveURLLikeModuleSpecifier(aSpecifier, baseURL);
+  nsCOMPtr<nsIURI> asURL;
+  if (parseResult.isOk()) {
+    asURL = parseResult.unwrap();
+  }
 
   if (aImportMap) {
-    // Step 5. Let baseURLString be baseURL, serialized.
+    // Step 6. Let baseURLString be baseURL, serialized.
     nsCString baseURLString = baseURL->GetSpecOrDefault();
 
-    // Step 7. Let normalizedSpecifier be the serialization of asURL, if asURL
+    // Step 8. Let normalizedSpecifier be the serialization of asURL, if asURL
     // is non-null; otherwise, specifier.
     nsAutoString normalizedSpecifier =
         asURL ? NS_ConvertUTF8toUTF16(asURL->GetSpecOrDefault())
               : nsAutoString{aSpecifier};
 
-    // Step 8. For each scopePrefix → scopeImports of importMap’s scopes,
+    // Step 9. For each scopePrefix → scopeImports of importMap’s scopes,
     for (auto&& [scopePrefix, scopeImports] : *aImportMap->mScopes) {
-      // Step 8.1. If scopePrefix is baseURLString, or if scopePrefix ends with
-      // U+002F (/) and baseURLString starts with scopePrefix, then:
+      // Step 9.1. If scopePrefix is baseURLString, or if scopePrefix ends with
+      // U+002F (/) and scopePrefix is a code unit prefix of baseURLString,
+      // then:
       if (scopePrefix.Equals(baseURLString) ||
           (StringEndsWith(scopePrefix, "/"_ns) &&
            StringBeginsWith(baseURLString, scopePrefix))) {
-        // Step 8.1.1. Let scopeImportsMatch be the result of resolving an
+        // Step 9.1.1. Let scopeImportsMatch be the result of resolving an
         // imports match given normalizedSpecifier, asURL, and scopeImports.
         auto result =
             ResolveImportsMatch(normalizedSpecifier, asURL, scopeImports.get());
@@ -615,7 +644,7 @@ ResolveResult ImportMap::ResolveModuleSpecifier(ImportMap* aImportMap,
         }
 
         nsCOMPtr<nsIURI> scopeImportsMatch = result.unwrap();
-        // Step 8.1.2. If scopeImportsMatch is not null, then return
+        // Step 9.1.2. If scopeImportsMatch is not null, then return
         // scopeImportsMatch.
         if (scopeImportsMatch) {
           LOG((
@@ -626,7 +655,7 @@ ResolveResult ImportMap::ResolveModuleSpecifier(ImportMap* aImportMap,
       }
     }
 
-    // Step 9. Let topLevelImportsMatch be the result of resolving an imports
+    // Step 10. Let topLevelImportsMatch be the result of resolving an imports
     // match given normalizedSpecifier, asURL, and importMap’s imports.
     auto result = ResolveImportsMatch(normalizedSpecifier, asURL,
                                       aImportMap->mImports.get());
@@ -635,7 +664,7 @@ ResolveResult ImportMap::ResolveModuleSpecifier(ImportMap* aImportMap,
     }
     nsCOMPtr<nsIURI> topLevelImportsMatch = result.unwrap();
 
-    // Step 10. If topLevelImportsMatch is not null, then return
+    // Step 11. If topLevelImportsMatch is not null, then return
     // topLevelImportsMatch.
     if (topLevelImportsMatch) {
       LOG(("ImportMap::ResolveModuleSpecifier returns topLevelImportsMatch: %s",
@@ -644,7 +673,7 @@ ResolveResult ImportMap::ResolveModuleSpecifier(ImportMap* aImportMap,
     }
   }
 
-  // Step 11. At this point, the specifier was able to be turned in to a URL,
+  // Step 12. At this point, the specifier was able to be turned in to a URL,
   // but it wasn’t remapped to anything by importMap. If asURL is not null, then
   // return asURL.
   if (asURL) {
@@ -653,8 +682,13 @@ ResolveResult ImportMap::ResolveModuleSpecifier(ImportMap* aImportMap,
     return WrapNotNull(asURL);
   }
 
-  // Step 12. Throw a TypeError indicating that specifier was a bare specifier,
+  // Step 13. Throw a TypeError indicating that specifier was a bare specifier,
   // but was not remapped to anything by importMap.
+  if (parseResult.unwrapErr() != ResolveError::FailureMayBeBare) {
+    // We may have failed to parse a non-bare specifier for another reason.
+    return Err(ResolveError::Failure);
+  }
+
   return Err(ResolveError::InvalidBareSpecifier);
 }
 
