@@ -1,12 +1,12 @@
-// |reftest| skip-if(!Object.prototype.toSource||(!xulRuntime.shell&&((isDebugBuild&&xulRuntime.OS=="Linux")||Android||xulRuntime.XPCOMABI.match(/x86_64/)||xulRuntime.OS=="WINNT"))) slow -- can fail silently due to out of memory, bug 621348 - timeouts on slow debug Linux
+// |reftest| skip-if(!Object.prototype.toSource||(!xulRuntime.shell&&((Android||(isDebugBuild&&gtkWidget)||is64Bit)))) slow -- can fail silently due to out of memory, bug 615011 - timeouts on slow debug Linux
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 336410;
-var summary = 'Integer overflow in array_toSource';
+var BUGNUMBER = 336409;
+var summary = 'Integer overflow in js_obj_toSource';
 var actual = 'No Crash';
 var expect = /(No Crash|InternalError: allocation size overflow|out of memory)/;
 
@@ -31,11 +31,11 @@ function createString(n)
 try
 {
   var n = 128;
-  printStatus('Creating ' + n + 'M length string');
+  printStatus('Creating ' + n + 'MB string');
   var r = createString(n);
   printStatus('Done. length = ' + r.length);
-  printStatus('Creating array');
-  var o=[r, r, r, r, r, r, r, r, r];
+  printStatus('Creating object');
+  var o = {f1: r, f2: r, f3: r,f4: r,f5: r, f6: r, f7: r, f8: r,f9: r};
   printStatus('object.toSource()');
   var rr = o.toSource();
   printStatus('Done.');
