@@ -11,7 +11,6 @@
 #include "mozilla/Bootstrap.h"
 #include "XREShellData.h"
 
-#include "nsXULAppAPI.h"
 #ifdef XP_MACOSX
 #  include "xpcshellMacUtils.h"
 #endif
@@ -78,6 +77,9 @@ int main(int argc, char** argv, char** envp) {
 #ifdef LIBFUZZER
   shellData.fuzzerDriver = fuzzer::FuzzerDriver;
 #endif
+#ifdef AFLFUZZ
+  shellData.fuzzerDriver = afl_interface_raw;
+#endif
 
   int result = bootstrap->XRE_XPCShellMain(argc, argv, envp, &shellData);
 
