@@ -7,14 +7,15 @@
 #ifndef __SANDBOXPRIVATE_H__
 #define __SANDBOXPRIVATE_H__
 
-#include "mozilla/WeakPtr.h"
+#include "mozilla/SchedulerGroup.h"
 #include "mozilla/StaticPrefs_dom.h"
 #include "mozilla/StorageAccess.h"
+#include "mozilla/WeakPtr.h"
 #include "mozilla/net/CookieJarSettings.h"
+#include "nsContentUtils.h"
 #include "nsIGlobalObject.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIPrincipal.h"
-#include "nsGlobalWindowInner.h"
 #include "nsWeakReference.h"
 #include "nsWrapperCache.h"
 
@@ -23,15 +24,15 @@
 #include "js/Object.h"  // JS::GetPrivate, JS::SetPrivate
 #include "js/RootingAPI.h"
 
-class SandboxPrivate : public nsIGlobalObject,
-                       public nsIScriptObjectPrincipal,
-                       public nsSupportsWeakReference,
-                       public mozilla::SupportsWeakPtr,
-                       public nsWrapperCache {
+class SandboxPrivate final : public nsIGlobalObject,
+                             public nsIScriptObjectPrincipal,
+                             public nsSupportsWeakReference,
+                             public mozilla::SupportsWeakPtr,
+                             public nsWrapperCache {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_AMBIGUOUS(SandboxPrivate,
-                                                         nsIGlobalObject)
+  NS_DECL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS_AMBIGUOUS(SandboxPrivate,
+                                                        nsIGlobalObject)
 
   static void Create(nsIPrincipal* principal, JS::Handle<JSObject*> global) {
     RefPtr<SandboxPrivate> sbp = new SandboxPrivate(principal);
@@ -43,6 +44,10 @@ class SandboxPrivate : public nsIGlobalObject,
     nsIScriptObjectPrincipal* sop =
         static_cast<nsIScriptObjectPrincipal*>(sbp.forget().take());
     JS::SetObjectISupports(global, sop);
+
+    JS::SetRealmReduceTimerPrecisionCallerType(
+        js::GetNonCCWObjectRealm(global),
+        RTPCallerTypeToToken(GetPrivate(global)->GetRTPCallerType()));
   }
 
   static SandboxPrivate* GetPrivate(JSObject* obj) {
@@ -56,6 +61,8 @@ class SandboxPrivate : public nsIGlobalObject,
 
   nsIPrincipal* GetPrincipal() override { return mPrincipal; }
 
+  nsIPrincipal* GetEffectiveCookiePrincipal() override { return mPrincipal; }
+
   nsIPrincipal* GetEffectiveStoragePrincipal() override { return mPrincipal; }
 
   nsIPrincipal* PartitionedPrincipal() override { return mPrincipal; }
@@ -79,6 +86,13 @@ class SandboxPrivate : public nsIGlobalObject,
 
   void ForgetGlobalObject(JSObject* obj) { ClearWrapper(obj); }
 
+  nsISerialEventTarget* SerialEventTarget() const final {
+    return mozilla::GetMainThreadSerialEventTarget();
+  }
+  nsresult Dispatch(already_AddRefed<nsIRunnable>&& aRunnable) const final {
+    return mozilla::SchedulerGroup::Dispatch(std::move(aRunnable));
+  }
+
   virtual JSObject* WrapObject(JSContext* cx,
                                JS::Handle<JSObject*> aGivenProto) override {
     MOZ_CRASH("SandboxPrivate doesn't use DOM bindings!");
@@ -86,11 +100,21 @@ class SandboxPrivate : public nsIGlobalObject,
 
   JS::loader::ModuleLoaderBase* GetModuleLoader(JSContext* aCx) override;
 
+  mozilla::Result<mozilla::ipc::PrincipalInfo, nsresult> GetStorageKey()
+      override;
+
   size_t ObjectMoved(JSObject* obj, JSObject* old) {
     UpdateWrapper(obj, old);
     return 0;
   }
 
+  bool ShouldResistFingerprinting(RFPTarget aTarget) const override {
+    return nsContentUtils::ShouldResistFingerprinting(
+        "Presently we don't have enough context to make an informed decision"
+        "on JS Sandboxes. See 1782853",
+        aTarget);
+  }
+
  private:
   explicit SandboxPrivate(nsIPrincipal* principal) : mPrincipal(principal) {}
 
